searchData={"items":[{"type":"module","title":"Circuits.GPIO","doc":"Control GPIOs from Elixir\n\nIf you're coming from Elixir/ALE, check out our [porting guide](PORTING.md).\n\n`Circuits.GPIO` works great with LEDs, buttons, many kinds of sensors, and\nsimple control of motors. In general, if a device requires high speed\ntransactions or has hard real-time constraints in its interactions, this is not\nthe right library. For those devices, see if there's a Linux kernel driver.","ref":"Circuits.GPIO.html"},{"type":"function","title":"Circuits.GPIO.close/1","doc":"Release the resources associated with the GPIO.\n\nThis is optional. The garbage collector will free GPIO resources that aren't in\nuse, but this will free them sooner.","ref":"Circuits.GPIO.html#close/1"},{"type":"function","title":"Circuits.GPIO.info/0","doc":"Return info about the low level GPIO interface\n\nThis may be helpful when debugging issues.","ref":"Circuits.GPIO.html#info/0"},{"type":"function","title":"Circuits.GPIO.open/3","doc":"Open a GPIO for use.\n\n`pin` should be a valid GPIO pin number on the system and `pin_direction`\nshould be `:input` or `:output`. If opening as an output, then be sure to set\nthe `:initial_value` option if you need the set to be glitch free.\n\nOptions:\n\n* :initial_value - Set to `:not_set`, `0` or `1` if this is an output.\n  `:not_set` is the default.\n* :pull_mode - Set to `:not_set`, `:pullup`, `:pulldown`, or `:none` for an\n   input pin. `:not_set` is the default.","ref":"Circuits.GPIO.html#open/3"},{"type":"function","title":"Circuits.GPIO.pin/1","doc":"Get the GPIO pin number","ref":"Circuits.GPIO.html#pin/1"},{"type":"function","title":"Circuits.GPIO.read/1","doc":"Read the current value on a pin.","ref":"Circuits.GPIO.html#read/1"},{"type":"function","title":"Circuits.GPIO.set_direction/2","doc":"Change the direction of the pin.","ref":"Circuits.GPIO.html#set_direction/2"},{"type":"function","title":"Circuits.GPIO.set_interrupts/3","doc":"Enable or disable pin value change notifications. The notifications\nare sent based on the trigger parameter:\n\n* :none - No notifications are sent\n* :rising - Send a notification when the pin changes from 0 to 1\n* :falling - Send a notification when the pin changes from 1 to 0\n* :both - Send a notification on all changes\n\nAvailable Options:\n* `suppress_glitches` - It is possible that the pin transitions to a value\nand back by the time that Circuits GPIO gets to process it. This controls\nwhether a notification is sent. Set this to `false` to receive notifications.\n* `receiver` - Process which should receive the notifications.\nDefaults to the calling process (`self()`)\n\nNotifications look like:\n\n```\n{:circuits_gpio, pin_number, timestamp, value}\n```\n\nWhere `pin_number` is the pin that changed values, `timestamp` is roughly when\nthe transition occurred in nanoseconds since host system boot time,\nand `value` is the new value.\n\nNOTE: You will need to store the `Circuits.GPIO` reference somewhere (like\nyour `GenServer`'s state) so that it doesn't get garbage collected. Event\nmessages stop when it gets collected. If you only get one message and you are\nexpecting more, this is likely the case.","ref":"Circuits.GPIO.html#set_interrupts/3"},{"type":"function","title":"Circuits.GPIO.set_pull_mode/2","doc":"Enable or disable internal pull-up or pull-down resistor to GPIO pin","ref":"Circuits.GPIO.html#set_pull_mode/2"},{"type":"function","title":"Circuits.GPIO.write/2","doc":"Set the value of a pin. The pin should be configured to an output\nfor this to work.","ref":"Circuits.GPIO.html#write/2"},{"type":"type","title":"Circuits.GPIO.open_option/0","doc":"Options for open/3","ref":"Circuits.GPIO.html#t:open_option/0"},{"type":"type","title":"Circuits.GPIO.pin_direction/0","doc":"The GPIO direction (input or output)","ref":"Circuits.GPIO.html#t:pin_direction/0"},{"type":"type","title":"Circuits.GPIO.pin_number/0","doc":"A GPIO pin number. See your device's documentation for how these connect to wires","ref":"Circuits.GPIO.html#t:pin_number/0"},{"type":"type","title":"Circuits.GPIO.pull_mode/0","doc":"Pull mode for platforms that support controllable pullups and pulldowns","ref":"Circuits.GPIO.html#t:pull_mode/0"},{"type":"type","title":"Circuits.GPIO.trigger/0","doc":"Trigger edge for pin change notifications","ref":"Circuits.GPIO.html#t:trigger/0"},{"type":"type","title":"Circuits.GPIO.value/0","doc":"GPIO logic value (low = 0 or high = 1)","ref":"Circuits.GPIO.html#t:value/0"},{"type":"extras","title":"Elixir Circuits - GPIO","doc":"# Elixir Circuits - GPIO\n\n[![Hex version](https://img.shields.io/hexpm/v/circuits_gpio.svg \"Hex version\")](https://hex.pm/packages/circuits_gpio)\n[![API docs](https://img.shields.io/hexpm/v/circuits_gpio.svg?label=hexdocs \"API docs\")](https://hexdocs.pm/circuits_gpio/Circuits.GPIO.html)\n[![CircleCI](https://circleci.com/gh/elixir-circuits/circuits_gpio.svg?style=svg)](https://circleci.com/gh/elixir-circuits/circuits_gpio)\n[![REUSE status](https://api.reuse.software/badge/github.com/elixir-circuits/circuits_gpio)](https://api.reuse.software/info/github.com/elixir-circuits/circuits_gpio)\n\n`Circuits.GPIO` lets you control or read from GPIOs on Nerves or other Linux-based\ndevices.\n\n*This is the v1 maintenance branch. This is the version you want for production\nuse now.  To follow v2.0 development, see the [circuits_gpio v2.0\nbranch](https://github.com/elixir-circuits/circuits_gpio/tree/v2.0)*\n\nIf you're coming from Elixir/ALE, check out our [porting guide](PORTING.md).\n\n`Circuits.GPIO` works great with LEDs, buttons, many kinds of sensors, and\nsimple control of motors. In general, if a device requires high speed\ntransactions or has hard real-time constraints in its interactions, this is not\nthe right library. For those devices, see if there's a Linux kernel driver.","ref":"readme.html"},{"type":"extras","title":"Getting started - Elixir Circuits - GPIO","doc":"If you're natively compiling `circuits_gpio` on a Raspberry Pi or using Nerves,\neverything should work like any other Elixir library. Normally, you would\ninclude `circuits_gpio` as a dependency in your `mix.exs` like this:\n\n```elixir\ndef deps do\n  [{:circuits_gpio, \"~> 1.0\"}]\nend\n```\n\nOne common error on Raspbian is that the Erlang headers are missing (`ie.h`),\nyou may need to install erlang with `apt-get install erlang-dev` or build Erlang\nfrom source per instructions [here](http://elinux.org/Erlang).","ref":"readme.html#getting-started"},{"type":"extras","title":"Examples - Elixir Circuits - GPIO","doc":"`Circuits.GPIO` only supports simple uses of the GPIO interface in Linux, but\nyou can still do quite a bit. The following examples were tested on a Raspberry\nPi that was connected to an [Erlang Embedded Demo\nBoard](http://solderpad.com/omerk/erlhwdemo/). There's nothing special about\neither the demo board or the Raspberry Pi, so these should work similarly on\nother embedded Linux platforms.","ref":"readme.html#examples"},{"type":"extras","title":"GPIO - Elixir Circuits - GPIO","doc":"A [General Purpose\nInput/Output](https://en.wikipedia.org/wiki/General-purpose_input/output) (GPIO)\nis just a wire that you can use as an input or an output. It can only be one of\ntwo values, 0 or 1. A 1 corresponds to a logic high voltage like 3.3 V and a 0\ncorresponds to 0 V. The actual voltage depends on the hardware.\n\nHere's an example of turning an LED on or off:\n\n![GPIO LED schematic](assets/images/schematic-gpio-led.png)\n\nTo turn on the LED that's connected to the net (or wire) labeled `GPIO18`, run\nthe following:\n\n```elixir\niex> {:ok, gpio} = Circuits.GPIO.open(18, :output)\n{:ok, #Reference }\n\niex> Circuits.GPIO.write(gpio, 1)\n:ok\n\niex> Circuits.GPIO.close(gpio)\n:ok\n```\n\n_Note that the call to `Circuits.GPIO.close/1` is not necessary, as the garbage\ncollector will free up any unreferenced GPIOs. It can be used to explicitly\nde-allocate connections you know you will not need anymore._\n\nInput works similarly. Here's an example of a button with a pull down resistor\nconnected.\n\n![GPIO Button schematic](assets/images/schematic-gpio-button.png)\n\nIf you're not familiar with pull up or pull down resistors, they're resistors\nwhose purpose is to drive a wire high or low when the button isn't pressed. In\nthis case, it drives the wire low. Many processors have ways of configuring\ninternal resistors to accomplish the same effect without needing to add an\nexternal resistor. If you're using a Raspberry Pi, you can use [the built-in\npull-up/pull-down resistors](#internal-pull-uppull-down).\n\nThe code looks like this in `Circuits.GPIO`:\n\n```elixir\niex> {:ok, gpio} = Circuits.GPIO.open(17, :input)\n{:ok, #Reference }\n\niex> Circuits.GPIO.read(gpio)\n0\n\n# Push the button down\n\niex> Circuits.GPIO.read(gpio)\n1\n```\n\nIf you'd like to get a message when the button is pressed or released, call the\n`set_interrupts` function. You can trigger on the `:rising` edge, `:falling` edge\nor `:both`.\n\n```elixir\niex> Circuits.GPIO.set_interrupts(gpio, :both)\n:ok\n\niex> flush\n{:circuits_gpio, 17, 1233456, 1}\n{:circuits_gpio, 17, 1234567, 0}\n:ok\n```\n\nNote that after calling `set_interrupts`, the calling process will receive an\ninitial message with the state of the pin. This prevents the race condition\nbetween getting the initial state of the pin and turning on interrupts. Without\nit, you could get the state of the pin, it could change states, and then you\ncould start waiting on it for interrupts. If that happened, you would be out of\nsync.\n\n#","ref":"readme.html#gpio"},{"type":"extras","title":"Internal pull-up/pull-down - Elixir Circuits - GPIO","doc":"To connect or disconnect an internal [pull-up or pull-down resistor](https://github.com/raspberrypilearning/physical-computing-guide/blob/master/pull_up_down.md) to a GPIO\npin, call the `set_pull_mode` function.\n\n```elixir\niex> Circuits.GPIO.set_pull_mode(gpio, pull_mode)\n:ok\n```\n\nValid `pull_mode` values are `:none` `:pullup`, or `:pulldown`\n\nNote that `set_pull_mode` is platform dependent, and currently only works for\nRaspberry Pi hardware.  Calls to `set_pull_mode` on other platforms will have no\neffect.  The internal pull-up resistor value is between 50K and 65K, and the\npull-down is between 50K and 60K.  It is not possible to read back the current\nPull-up/down settings, and GPIO pull-up pull-down resistor connections are\nmaintained, even when the CPU is powered down.\n\nTo get the GPIO pin number for a gpio reference, call the `pin` function.\n\n```elixir\niex> Circuits.GPIO.pin(gpio)\n17\n```","ref":"readme.html#internal-pull-up-pull-down"},{"type":"extras","title":"Testing - Elixir Circuits - GPIO","doc":"`Circuits.GPIO` supports a \"stub\" hardware abstraction layer on platforms\nwithout GPIO support and when `MIX_ENV=test`. The stub allows for some\nlimited unit testing without real hardware.\n\nTo use it, first check that you're using the \"stub\" HAL:\n\n```elixir\niex> Circuits.GPIO.info\n%{name: :stub, pins_open: 0}\n```\n\nThe stub HAL has 64 GPIOs. Each pair of GPIOs is connected. For example,\nGPIO 0 is connected to GPIO 1. If you open GPIO 0 as an output and\nGPIO 1 as an input, you can write to GPIO 0 and see the result on GPIO 1.\nHere's an example:\n\n```elixir\niex> {:ok, gpio0} = Circuits.GPIO.open(0, :output)\n{:ok, #Reference }\niex> {:ok, gpio1} = Circuits.GPIO.open(1, :input)\n{:ok, #Reference }\niex> Circuits.GPIO.read(gpio1)\n0\niex> Circuits.GPIO.write(gpio0, 1)\n:ok\niex> Circuits.GPIO.read(gpio1)\n1\n```\n\nThe stub HAL is fairly limited, but it does support interrupts.\n\nIf `Circuits.GPIO` is used as a dependency the stub may not be present. To\nmanually enable it, set `CIRCUITS_MIX_ENV` to `test` and rebuild\n`circuits_gpio`.","ref":"readme.html#testing"},{"type":"extras","title":"FAQ - Elixir Circuits - GPIO","doc":"#","ref":"readme.html#faq"},{"type":"extras","title":"How does Circuits.GPIO compare to Elixir/ALE? - Elixir Circuits - GPIO","doc":"`Circuits.GPIO` is almost Elixir/ALE 2.0. The API for Elixir/ALE became\ndifficult to change so we started again with `Circuits.GPIO`. The main\nimprovements are:\n\n* Improved performance and lower resource usage\n* Timestamps on interrupts\n* A hardware abstraction layer to support multiple ways of interfacing with the\n  low level GPIO interfaces\n\nSee the [Porting Guide](PORTING.md) for more information if you're an Elixir/ALE\nuser.\n\n#","ref":"readme.html#how-does-circuits-gpio-compare-to-elixir-ale"},{"type":"extras","title":"Where can I get help? - Elixir Circuits - GPIO","doc":"Most issues people have are on how to communicate with hardware for the first\ntime. Since `Circuits.GPIO` is a thin wrapper on the Linux sys class interface,\nyou may find help by searching for similar issues when using Python or C.\n\nFor help specifically with `Circuits.GPIO`, you may also find help on the nerves\nchannel on the [elixir-lang Slack](https://elixir-slackin.herokuapp.com/).  Many\n[Nerves](http://nerves-project.org) users also use `Circuits.GPIO`.\n\n#","ref":"readme.html#where-can-i-get-help"},{"type":"extras","title":"I tried turning on and off a GPIO as fast as I could. Why was it slow? - Elixir Circuits - GPIO","doc":"Please don't do that - there are so many better ways of accomplishing whatever\nyou're trying to do:\n\n1. If you're trying to drive a servo or dim an LED, look into PWM. Many\n   platforms have PWM hardware and you won't tax your CPU at all. If your\n   platform is missing a PWM, several chips are available that take I2C commands\n   to drive a PWM output.\n2. If you need to implement a wire level protocol to talk to a device, look for\n   a Linux kernel driver. It may just be a matter of loading the right kernel\n   module.\n3. If you want a blinking LED to indicate status, `gpio` really should\n   be fast enough to do that, but check out Linux's LED class interface. Linux\n   can flash LEDs, trigger off events and more. See [nerves_leds](https://github.com/nerves-project/nerves_leds).\n\nIf you're still intent on optimizing GPIO access, you may be interested in\n[gpio_twiddler](https://github.com/fhunleth/gpio_twiddler).\n\n#","ref":"readme.html#i-tried-turning-on-and-off-a-gpio-as-fast-as-i-could-why-was-it-slow"},{"type":"extras","title":"Can I develop code that uses GPIO on my laptop? - Elixir Circuits - GPIO","doc":"See whether the \"stub\" HAL (described above) works for you or could be improved\nto support your use case.\n\nThe following advice from Elixir/ALE may also be useful: You'll need to fake out\nthe hardware. Code to do this depends on what your hardware actually does, but\nhere's one example:\n\n* [Compiling and testing Elixir Nerves on your host machine](https://cultivatehq.com/posts/compiling-and-testing-elixir-nerves-on-your-host-machine/)","ref":"readme.html#can-i-develop-code-that-uses-gpio-on-my-laptop"},{"type":"extras","title":"License - Elixir Circuits - GPIO","doc":"All original source code in this project is licensed under Apache-2.0.\n\nAdditionally, this project follows the [REUSE recommendations](https://reuse.software)\nand labels so that licensing and copyright are clear at the file level.\n\nExceptions to Apache-2.0 licensing are:\n\n* Configuration and data files are licensed under CC0-1.0\n* Documentation files are CC-BY-4.0\n* Erlang Embedded board images are Solderpad Hardware License v0.51.","ref":"readme.html#license"},{"type":"extras","title":"Porting Elixir/ALE to Circuits.GPIO","doc":"# Porting Elixir/ALE to Circuits.GPIO\n\nThe `Circuits.GPIO` package is the next version of Elixir/ALE's GPIO support.\nIf you're currently using Elixir/ALE, you're encouraged to switch. Here are some\nbenefits:\n\n1. Supported by both the maintainer of Elixir/ALE and a couple others. They'd\n   prefer to support `Circuits.GPIO` issues.\n2. Much faster than Elixir/ALE - like it's not even close. The guideline is\n   still to use GPIOs for buttons, LEDs, and other low frequency devices and use\n   specialized device drivers for the rest. However, we know that's not always easy\n   and the extra performance is really nice to have.\n3. Included pull up/pull down support for Raspberry Pi. We'd like to include\n   this for other platforms as well. This was the single-most requested feature\n   for Elixir/ALE.\n4. Timestamped interrupt reports - This is makes it possible to measure time\n   deltas between GPIO changes with higher accuracy by removing variability from\n   messages sitting in queues and passing between processes\n5. Interrupt glitch detection  - When the GPIO toggles faster than\n   `Circuits.GPIO` can handle, it can either synthesize an interrupt event or\n   suppress the event.\n6. Lower resource usage - Elixir/ALE created a GenServer and OS process for each\n   GPIO. `Circuits.GPIO` creates a NIF resource (Elixir Reference) for each\n   GPIO.\n\n`Circuits.GPIO` uses Erlang's NIF interface. NIFs have the downside of being\nable to crash the Erlang VM. Experience with Elixir/ALE has given many of us\nconfidence that this won't be a problem.","ref":"porting.html"},{"type":"extras","title":"Code modifications - Porting Elixir/ALE to Circuits.GPIO","doc":"`Circuits.GPIO` is not a `GenServer`, so if you've added `ElixirALE.GPIO` to a\nsupervision tree, you'll have to take it out and manually call\n`Circuits.GPIO.open` to obtain a reference. A common pattern is to create a\n`GenServer` that wraps the GPIO and is descriptive of what the GPIO controls or\nsignals. Put the `Circuits.GPIO.open` call in your `init/1` callback.\n\nThe remain modifications should mostly be mechanical:\n\n1. Rename references to `ElixirALE.GPIO` to `Circuits.GPIO` and `elixir_ale`\n   to `circuits_gpio`\n2. Change calls to `ElixirALE.GPIO.start_link/2` to `Circuits.GPIO.open/2`.\n   While you're at it, review the arguments to open to not include any\n   `GenServer` options.\n3. Change calls to `ElixirALE.GPIO.set_int/2` to\n   `Circuits.GPIO.set_interrupts/3`.\n4. Change the pattern match for the GPIO interrupt events to match 4 tuples.\n   They have the form `{:circuits_gpio,  ,  ,  }`\n5. Review calls to `write/2` to ensure that they pass `0` or `1`. `ElixirALE`\n   allowed users to pass `true` and `false`. That won't work. Running Dialyzer\n   should catch this change as well.\n6. Consider adding a call to `Circuits.GPIO.close/1` if there's an obvious place\n   to release the GPIO. This is not strictly necessary since the garbage\n   collector will free unreferenced GPIOs.\n\nIf you find that you have to make any other changes, please let us know via an\nissue or PR so that other users can benefit.","ref":"porting.html#code-modifications"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.2.0 - 2023-12-14 - Changelog","doc":"* Changes\n  * Add workaround to GPIO pin numbering change on TI AM335x platforms (like the\n    BBB) that happened between Linux 5.10 and 5.15. Circuits.GPIO will remap the\n    pins automatically so the number is consistent with the way it's been for\n    years and the way that most online docs talk about it. E.g., GPIO number is\n    the pin bank times 32 plus the pin offset in the bank.\n  * Defer NIF loading until opening the first GPIO. This is a minor optimization\n    to delay shared library load time and issue to a little later in boot.","ref":"changelog.html#v1-2-0-2023-12-14"},{"type":"extras","title":"v1.1.0 - 2022-12-31 - Changelog","doc":"* Changes\n  * Remove Erlang convenience functions since no one used them\n  * Require Elixir 1.10 or later. Previous versions probably work, but won't be\n    supported. This opens up the possibility of using Elixir 1.10+ features in\n    future releases.","ref":"changelog.html#v1-1-0-2022-12-31"},{"type":"extras","title":"v1.0.1 - 2022-7-26 - Changelog","doc":"* Bug fixes\n  * On at least one device, the pin direction reported by Linux does not match\n    the actual pin direction. This release forces the pin direction for inputs.\n    It cannot do this for outputs, since setting a pin to output has a side\n    effect on the pin state. This technically is a bug in a Linux driver port,\n    but it was harmless to workaround for inputs so that's what's done. Thanks\n    to @pojiro for investigating and fixing this issue.","ref":"changelog.html#v1-0-1-2022-7-26"},{"type":"extras","title":"v1.0.0 - 2021-10-20 - Changelog","doc":"This release only changes the version number. No code has changed.","ref":"changelog.html#v1-0-0-2021-10-20"},{"type":"extras","title":"v0.4.8 - Changelog","doc":"This release only has doc and build output cleanup. No code has changed.","ref":"changelog.html#v0-4-8"},{"type":"extras","title":"v0.4.7 - Changelog","doc":"* Bug fixes\n  * Fix hang when unloading the NIF. This bug caused `:init.stop` to never\n    return and anything else that would try to unload this module.\n  * Fix C compiler warnings with OTP 24\n\nThe minimum Elixir version has been changed from 1.4 to 1.6. Elixir 1.4 might\nstill work, but it's no longer being verified on CI.","ref":"changelog.html#v0-4-7"},{"type":"extras","title":"v0.4.6 - Changelog","doc":"* Bug fixes\n  * Fix quoting issue that was causing failures on Yocto. Thanks to Zander\n    Erasmus for this.","ref":"changelog.html#v0-4-6"},{"type":"extras","title":"v0.4.5 - Changelog","doc":"* Bug fixes\n  * Opening a GPIO to read its state won't clear the interrupt status of another\n    listener. Registering for interrupts a second time will still clear out the\n    first registree, though. That's a limitation of the interface. However, for\n    debugging, it can be nice to look at a GPIO without affecting the program\n    and this change allows for that.","ref":"changelog.html#v0-4-5"},{"type":"extras","title":"v0.4.4 - Changelog","doc":"* Bug fixes\n  * Add -fPIC to compilation flags to fix build with `nerves_system_x86_64` and\n    other environments using the Musl C toolchains","ref":"changelog.html#v0-4-4"},{"type":"extras","title":"v0.4.3 - Changelog","doc":"* Bug fixes\n  * Fix GPIO glitch suppression when interrupts were enabled. Glitch suppression\n    filters out transitions on a GPIO line that are too fast for Linux and the\n    NIF to see both the rising and falling edges. Turning it off synthesizes\n    events. You can identify synthesized events since they have the same\n    timestamp to the nanosecond. See `Circuits.GPIO.set_interrupts/3`.\n\n* Improvement\n  * It's possible to enable the \"stub\" on Linux by setting\n    `CIRCUITS_MIX_ENV=test`. This can be useful for unit testing code that uses\n    Circuits.GPIO. Thanks to Enrico Rivarola for adding this!","ref":"changelog.html#v0-4-3"},{"type":"extras","title":"v0.4.2 - Changelog","doc":"* Bug fixes\n  * Fix pullup/pulldown support for the Raspberry Pi 4","ref":"changelog.html#v0-4-2"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"* Bug fixes\n  * Fix a race condition on Raspbian where Circuits.GPIO would try to open the\n    GPIO sysfs file before udev had a chance to fix its permissions.\n  * Fix RPi platform detection on Raspbian so that pull-ups/pull-downs work\n    without passing any flags.","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"The GPIO interrupt notification messages have been changed for consistency with\nother circuits projects. The initial element of the tuple is now\n`:circuits_gpio`, so messages will look like:\n\n`{:circuits_gpio, 19, 83268239, 1}`\n\nPlease update your project if you call `set_interrupts/2`.\n\nNo more backwards incompatible changes are expected until after 1.0.","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"* Bug fixes\n  * Build C source under the `_build` directory so that changing targets\n    properly rebuilds the C code as well as the Elixir.","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"* New features\n  * Support `pull_mode` initializion in `open/3`.","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"* New features\n  * Add support for opening GPIOs to an initial value or to not change the\n    value. This removes a glitch if you want the GPIO to start out high (the\n    default was low) or if you want the GPIO to keep its current value.","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release to hex.","ref":"changelog.html#v0-1-0"}],"content_type":"text/markdown"}